# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
messages presents a uniform layer to deal with messages from both SIM and DB
"""

__version__ = "$Rev: 600 $"

from twisted.internet import defer

from vmc.common.persistent import SMSManager, SIM_SIZE
from vmc.controllers.base import TV_DICT_REV

def is_sim_message(sms):
    """Returns True if C{contact} is a SIM contact"""
    return int(sms.index) < SIM_SIZE

class Messages(object):
    """I deal with all SMS in system"""
    def __init__(self, sconn):
        self.sconn = sconn
        self.smanager = SMSManager()
    
    def close(self):
        self.smanager.close()
        del self.smanager
        del self.sconn
    
    def add_messages(self, smslist, where=None):
        if where:
            return self.smanager.add_messages(smslist, where)
        
        deferred = defer.Deferred()
        response = []
        sms_cb = lambda resp: response.append(resp)
        list_cb = lambda _: deferred.callback(response)
        responses = [self.add_message(sms).addCallback(sms_cb)
                     for sms in smslist]
        deflist = defer.DeferredList(responses)
        deflist.addCallback(list_cb)
        return deferred
    
    def add_message(self, sms, where=TV_DICT_REV['drafts_treeview']):
        if where:
            return self.smanager.add_message(sms, where)
        
        if is_sim_message(sms):
            raise NotImplementedError
    
    def get_messages(self):
        deferred = defer.Deferred()
        def sim_cb(simsms):
            def final_cb(dbsms):
                deferred.callback(simsms + dbsms)
            
            self.smanager.read_messages().addCallback(final_cb)
        
        self.sconn.get_all_sms().addCallback(sim_cb)
        return deferred
    
    def get_message(self, sms):
        if is_sim_message(sms):
            return self.sconn.get_sms(sms.index)
        else:
            raise NotImplementedError
    
    def delete_messages(self, smslist):
        responses = []
        for sms in smslist:
            if is_sim_message(sms):
                responses.append(self.sconn.delete_sms(sms.index))
            else:
                responses.append(self.smanager.delete_messages([sms]))
        
        return defer.DeferredList(responses)
    