# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
This module emulates the plugin passwordfd.so

In a less secure manner of course, as passwordfd.so uses a pipe and we blindly
overwrite the file. 
"""
__version__ = "$Rev: 658 $"

import re

#from vmc.common.oal import mark_for_oal

class SecretsEntry(object):
    def __init__(self, user, host, password, ip=None):
        self.user = user
        self.host = host
        self.password = password
        self.ip = ip
    
    def __eq__(self, entry):
        if self.user == entry.user and self.host == entry.host \
                    and self.password == self.password:
            try:
                if self.ip == entry.ip:
                    return True
                return False
            except NameError:
                return True
        
        return False
    
    def to_csv(self):
        if not self.ip:
            return '"%s"\t%s\t"%s"\n' % (self.user, self.host, self.password)
        else:
            return '"%s"\t%s\t"%s"\t%s\n' % (self.user, self.host,
                                           self.password, self.ip)
    
def append_username_passwd_to_secret(path, entry):
    # read contents
    lines = open(path).readlines()
    entries = extract_entries_from_lines(lines)
    
    if entry not in entries:
        # now paste contents back and we might add something
        fobj = open(path, 'w')
        fobj.writelines(lines)
        fobj.write(entry.to_csv())
        fobj.close()

def extract_entries_from_lines(lines):
    pattern = re.compile(r"""
            "?(?P<user>\S+)"?
            \t\*\t
            "?(?P<passwd>\S+)"?
            """, re.VERBOSE | re.M)
    response = []
    for line in lines:
        match = pattern.match(line)
        if match:
            user, passwd = match.groups()
            if user[-1] == '"':
                user = user[:-1]
            response.append(SecretsEntry(user, '*', passwd))
    
    return response

if __name__ == '__main__':
    entry = SecretsEntry('vfpruebas', '*', 'vodafone')
    append_username_passwd_to_secret('/etc/ppp/pap-secrets', entry)
    