# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""Model for SMS-related controllers"""
__version__ = "$Rev: 600 $"

import os

import gtk
import gobject

from vmc.gtkmvc import ListStoreModel, Model
import vmc.utils.globals
from vmc.common.phonebook import PhoneBook

MOBILE_IMG = gtk.gdk.pixbuf_new_from_file(
              os.path.join(vmc.utils.globals.IMAGES_DIR, 'mobile.png'))
COMPUTER_IMG = gtk.gdk.pixbuf_new_from_file(
              os.path.join(vmc.utils.globals.IMAGES_DIR, 'computer.png'))

class NewSmsModel(Model):
    __properties__ = {}
    
    def __init__(self):
        Model.__init__(self)

class SMSStoreModel(ListStoreModel):
    """
    ListStoreModel beefed-up
    
    Adds a couple of convenience methods on top of ListStoreModel. Accepts
    a callable because as we live in a hotplugging world we need a way to
    obtain the device.
    """
    def __init__(self, _callable):
        ListStoreModel.__init__(self, gtk.gdk.Pixbuf,
                    gobject.TYPE_STRING, gobject.TYPE_STRING,
                    gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)
        self._callable = _callable
        self.sconn = None
    
    def add_messages(self, messages):
        for sms in messages:
            self.add_message(sms)
    
    def add_message(self, message):
        if not self.sconn:
            self.sconn = self._callable()
            
        phonebook = PhoneBook(self.sconn)
        
        def lookup_number_cb(contacts):
            entry = []
            if message.index < 1000:
                entry.append(MOBILE_IMG)
            else:
                entry.append(COMPUTER_IMG)
            
            entry.append(message.get_text())
            if contacts:
                entry.append(contacts[0].name)
            else:
                entry.append(message.get_number())
            
            entry.append(message.datetime)
            entry.append(message)
        
            self.append(entry)
            phonebook.close()
        
        d = phonebook.find_contact(number=message.get_number())
        d.addCallback(lookup_number_cb)
        